@@ -15,6 +15,14 @@
 /*  `final' properties to an input string of character codes for the Arabic */
 /*  script.                                                                 */
 /*                                                                          */
+/*  This file is part of the FreeType project, and may only be used         */
+/*  modified and distributed under the terms of the FreeType project        */
+/*  license, LICENSE.TXT.  By continuing to use, modify, or distribute      */
+/*  this file you indicate that you have read the license and               */
+/*  understand and accept it fully.                                         */
+/*                                                                          */
+/* The file LICENSE.TXT can be found in the Pango distribution as           */
+/* pango/opentype/FT-license.txt                                            */
 /****************************************************************************/
 
 #include "arabic-ot.h"
@@ -23,17 +31,17 @@
 /*
  *
  *     Here a table of the joining classes for characters in the range
- *     U+0620 - U+06FF.
+ *     U+0620 - U+06FF and U+0750 - U+077F.
  *
  *    The following character also has a joining class:
  *
- *     U+200C  ZERO WIDTH NON-JOINER  -> causing
+ *     U+200D  ZERO WIDTH JOINER  -> causing
  *
  *    All other characters are given the joining class `none'.
  *
  */
 
-joining_class  arabic[] =
+static const joining_class  arabic[] =
 {
   /* U+0620 */
   none, none, right, right,
@@ -50,23 +58,23 @@ joining_class  arabic[] =
   /* U+0640 */
   causing, dual, dual, dual,
   dual, dual, dual, dual,
-  right, right, dual, transparent,
+  right, dual, dual, transparent,
   transparent, transparent, transparent, transparent,
 
   /* U+0650 */
+  transparent, transparent, transparent, transparent,
+  transparent, transparent, transparent, transparent,
+  transparent, transparent, transparent, transparent,
   transparent, transparent, transparent, none,
-  none, none, none, none,
-  none, none, none, none,
-  none, none, none, none,
 
   /* U+0660 */
   none, none, none, none,
   none, none, none, none,
   none, none, none, none,
-  none, none, none, none,
+  none, none, dual, dual,
 
   /* U+0670 */
-  transparent, none, right, right,
+  transparent, right, right, right,
   none, right, right, right,
   dual, dual, dual, dual,
   dual, dual, dual, dual,
@@ -103,7 +111,7 @@ joining_class  arabic[] =
 
   /* U+06D0 */
   dual, dual, right, right,
-  none, none, none, transparent,
+  none, right, none, transparent,
   transparent, transparent, transparent, transparent,
   transparent, transparent, transparent, transparent,
 
@@ -111,13 +119,34 @@ joining_class  arabic[] =
   transparent, transparent, transparent, transparent,
   transparent, none, none, transparent,
   transparent, none, transparent, transparent,
-  transparent, transparent, none, none,
+  transparent, transparent, right, right,
 
   /* U+06F0 */
   none, none, none, none,
   none, none, none, none,
   none, none, dual, dual,
-  dual, none, none, none
+  dual, none, none, dual
+};
+
+static const joining_class  arabic_supplement[] =
+{
+  /* U+0750 */
+  dual, dual, dual, dual,
+  dual, dual, dual, dual,
+  dual, right, right, right,
+  dual, dual, dual, dual,
+
+  /* U+0760 */
+  dual, dual, dual, dual,
+  dual, dual, dual, dual,
+  dual, dual, dual, right,
+  right, dual, none, none,
+
+  /* U+0770 */
+  none, none, none, none,
+  none, none, none, none,
+  none, none, none, none,
+  none, none, none, none
 };
 
 #if 0
@@ -265,25 +294,25 @@ static joining_class  Get_Joining_Class (gunichar*   string,
 
       if (pos >= length)
         return none;
-      
-      if (string[pos] < 0x0620 ||
-	  string[pos] >= 0x0700)
-	{
-	  if (string[pos] == 0x200C)
-	    return causing;
-	  else
-	    return none;
-	}
+
+      if (string[pos] >= 0x0620 &&
+          string[pos] < 0x0700)
+        j = arabic[string[pos] - 0x0620];
+      else if (string[pos] >= 0x0750 &&
+          string[pos] < 0x0780)
+        j = arabic_supplement[string[pos] - 0x0750];
+      else if (string[pos] == 0x200D)
+        return causing;
       else
-        j =  arabic[string[pos] - 0x0620];
-      
+        return none;
+
       if (!direction || j != transparent)
         return j;
     }
 }
 
 
-FT_Error  Assign_Arabic_Properties (gunichar    *string,
+FT_Error  Arabic_Assign_Properties (gunichar    *string,
 				    gulong      *properties,
 				    int          length)
 {
